<?php

namespace DigiMember\Elementor\Modules\Widgets;

use Exception;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Typography;
use Elementor\Plugin;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;

class Signup extends Widget_Base
{
    public function get_name()
    {
        return 'dm_signup';
    }

    public function get_title()
    {
        return _dmelem('Anmeldeformular');
    }

    public function get_icon()
    {
        return 'icon-digimember';
    }

    public function get_keywords()
    {
        return ['digimember', 'signup', 'form'];
    }

    private function handle_request()
    {
        $settings = $this->get_settings_for_display();
        
        $email = ncore_retrieve($_POST, 'email');
        $first_name = ncore_retrieve($_POST, 'first_name');
        $last_name = ncore_retrieve($_POST, 'last_name');
        $is_confirmed = ncore_retrieve($_POST, 'is_confirmed');

        $validation_errors = $this->validate_request([
            'is_confirmed'  => $settings['require_checkbox'] == 'yes' && $settings['show_checkbox'] == 'yes' ? '' : 'required',
            'last_name'     => $settings['show_first_name'] == 'yes' ? 'required' : '',
            'first_name'    => $settings['show_last_name'] ? 'required' : '',
            'email'         => 'email|required',
        ]);

        if (count($validation_errors)) {
            $this->render_errors($validation_errors);
            return;
        }

        if ($settings['require_checkbox'] == 'yes' && !$is_confirmed) {
            $this->render_errors([
                _dgyou('Please accept our terms and check the checkbox.')
            ]);
            return;
        }
        if ($settings['require_checkbox'] == 'yes') {
            $text = $settings['label_checkbox'];
            dm_api()->log('privacy', _digi('User %s with IP %s has accepted our text: %s'), $email, ncore_clientIp(), $text);
        }

        $show_recaptcha = $settings['show_recaptcha'];
        $recaptcha_key = $settings['recaptcha_key'];
        $recaptcha_secret = $settings['recaptcha_secret'];
        if ($show_recaptcha == 'yes' && trim($recaptcha_key && $recaptcha_secret)) {
            $response = ncore_retrieve($_POST, 'g-recaptcha-response');
            
            $args = [];
            $args['body'] = 'secret=' . $recaptcha_secret . '&response=' . $response;

            $url = 'https://www.google.com/recaptcha/api/siteverify';

            $result = wp_remote_post($url, $args);

            $json = ncore_retrieve($result, 'body');
            $result = $json
                ? @json_decode($json)
                : false;

            $is_valid = ncore_retrieve($result, 'success', false);

            if (!$is_valid) {
                $this->render_errors([
                    _dgyou('Please prove that you are human and not a bot.')
                ]);
                return;
            }
        }

        $product_ids = join(',', $settings['products']);
        if (!$product_ids) {
            $product_ids = 'none';
        }
        $address = [
            'first_name' => $first_name,
            'last_name' => $last_name,
        ];

        $do_perform_login = $settings['auto_login'];

        $user_id = ncore_getUserIdByEmail($email);
        $hadAccount = $user_id > 0;
        $hadAdminAccount = false;

        if ($hadAccount) {
            $hadAdminAccount = ncore_canAdmin($user_id);
        } else {
            /** @var ncore_IpLockData $model */
            $model = dm_api()->load->model('data/ip_lock');
            $is_locked = $model->isLocked('signup', 10, 86400);
            if ($is_locked) {
                $this->render_errors([_dgyou('Sorry, you have created too many accounts already.')]);
                return;
            }
        }

        try {
            /** @var digimember_PaymentHandlerLib $library */
            $library = dm_api()->load->library('payment_handler');
            $welcomeMsgSent = $library->signUp($email, $product_ids, $address, $do_perform_login);

            ncore_flashMessage(NCORE_NOTIFY_SUCCESS, $this->successMessage($email, $hadAccount, $hadAdminAccount, $welcomeMsgSent));
        } catch (Exception $e) {
            $this->render_errors([$e->getMessage()]);
            return;
        }
    }

    private $has_post_errors = false;
    private function render_errors($errors)
    {
        if (count($errors)) {
            $this->has_post_errors = true;
        }
        foreach ($errors as $error) {
            if ($error) {
                ncore_flashMessage(NCORE_NOTIFY_ERROR, $error);
            }
        }
    }

    /**
     * @param string $email
     * @param bool   $hadAccount
     * @param bool   $hadAdminAccount
     * @param bool   $welcomeMsgSent
     *
     * @return string|void
     */
    private function successMessage($email, $hadAccount, $hadAdminAccount, $welcomeMsgSent)
    {
        if ($hadAdminAccount && ncore_canAdmin()) {
            return _digi('The email address %s belongs to an admin account. For testing purposes, please use a NON admin email address, because for admins wordpress looks and behaves different than for regular users.', "<em>$email</em>");
        } else if (!$welcomeMsgSent) {
            /** @var digimember_BlogConfigLogic $config */
            $config = dm_api()->load->model('logic/blog_config');
            $login_url = $config->loginUrl();

            $msg = _dgyou('There is already an account for the email address %s.', "<em>$email</em>");
            if ($login_url) {
                $msg .= ' ' . ncore_linkReplace(_dgyou('<a>Click here to login.</a>'), $login_url);
            }
            return $msg;
        } else if ($hadAccount) {
            return _dgyou('There is already an account for the email address %s. We have re-sent the confirmation email to this email address.', "<em>$email</em>");
        } else {
            return _dgyou('Your account has been created. We have send your password to %s. If you don\'t receive the email within 10 minutes, please check your spam folder.', "<em>$email</em>");
        }
    }

    /**
     * @return array
     */
    protected function validate_request($validation)
    {
        $errors = [];
        /** @var ncore_RuleValidatorLib $validator */
        $validator = dm_api()->load->library('rule_validator');

        foreach ($validation as $key => $rules) {
            $value = isset($_POST[$key]) ? $_POST[$key] : false;
            if ($value !== false) {
                $error = $validator->validate($key, $value, $rules);
                if ($error) {
                    $errors[$key] = $error;
                }
            }
        }
        return $errors;
    }

    /**
     * @return bool
     */
    private function is_posted()
    {
        if (empty($_POST) || !is_array($_POST) || !count($_POST)) {
            return false;
        }
        if (ncore_retrieve($_POST, 'dm_elem', '') != 'dm_elem_signup') {
            return false;
        }
        
        return true;
    }
    
    public function _register_controls()
    {
        $this->_register_general_controls();
        $this->_register_style_controls();
    }

    private function _register_style_controls()
    {
        $this->start_controls_section(
            'section_style',
            [
                'label' => _dmelem('Formular'),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_control(
            'row_gap',
            [
                'label' => _dmelem('Zeilenabstand'),
                'type' => Controls_Manager::SLIDER,
                'default' => [
                    'size' => '10',
                ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 60,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .elementor-form-fields-wrapper' => 'margin-bottom: -{{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'links_color',
            [
                'label' => _dmelem('Linkfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group a' => 'color: {{VALUE}};',
                ],
                'global' => [
                    'default' => Global_Colors::COLOR_TEXT,
                ],
            ]
        );

        $this->add_control(
            'links_hover_color',
            [
                'label' => _dmelem('Linkfarbe schwebend'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group a:hover' => 'color: {{VALUE}};',
                ],
                'global' => [
                    'default' => Global_Colors::COLOR_ACCENT,
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_style_labels',
            [
                'label' => _dmelem('Labels'),
                'tab' => Controls_Manager::TAB_STYLE,
                'condition' => [
                    'show_labels!' => '',
                ],
            ]
        );

        $this->add_control(
            'label_spacing',
            [
                'label' => _dmelem('Abstand'),
                'type' => Controls_Manager::SLIDER,
                'default' => [
                    'size' => '0',
                ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 60,
                    ],
                ],
                'selectors' => [
                    'body {{WRAPPER}} .elementor-field-group > label' => 'padding-bottom: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'label_color',
            [
                'label' => _dmelem('Textfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-form-fields-wrapper label' => 'color: {{VALUE}};',
                ],
                'global' => [
                    'default' => Global_Colors::COLOR_TEXT,
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'label_typography',
                'selector' => '{{WRAPPER}} .elementor-form-fields-wrapper label',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT,
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_field_style',
            [
                'label' => _dmelem('Felder'),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_control(
            'field_text_color',
            [
                'label' => __('Textfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group .elementor-field' => 'color: {{VALUE}};',
                ],
                'global' => [
                    'default' => Global_Colors::COLOR_TEXT,
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'field_typography',
                'selector' => '{{WRAPPER}} .elementor-field-group .elementor-field, {{WRAPPER}} .elementor-field-subgroup label',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT,
                ],
            ]
        );

        $this->add_control(
            'field_background_color',
            [
                'label' => _dmelem('Hintergrundfarbe'),
                'type' => Controls_Manager::COLOR,
                'default' => '#ffffff',
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group .elementor-field:not(.elementor-select-wrapper)' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} .elementor-field-group .elementor-select-wrapper select' => 'background-color: {{VALUE}};',
                ],
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'field_border_color',
            [
                'label' => _dmelem('Rahmenfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group .elementor-field:not(.elementor-select-wrapper)' => 'border-color: {{VALUE}};',
                    '{{WRAPPER}} .elementor-field-group .elementor-select-wrapper select' => 'border-color: {{VALUE}};',
                    '{{WRAPPER}} .elementor-field-group .elementor-select-wrapper::before' => 'color: {{VALUE}};',
                ],
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'field_border_width',
            [
                'label' => _dmelem('Rahmenbreite'),
                'type' => Controls_Manager::DIMENSIONS,
                'placeholder' => '1',
                'size_units' => ['px'],
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group .elementor-field:not(.elementor-select-wrapper)' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} .elementor-field-group .elementor-select-wrapper select' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'field_border_radius',
            [
                'label' => _dmelem('Rahmenrundung'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors' => [
                    '{{WRAPPER}} .elementor-field-group .elementor-field:not(.elementor-select-wrapper)' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} .elementor-field-group .elementor-select-wrapper select' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_button_style',
            [
                'label' => _dmelem('Button'),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->start_controls_tabs('tabs_button_style');

        $this->start_controls_tab(
            'tab_button_normal',
            [
                'label' => _dmelem('Normal'),
            ]
        );

        $this->add_control(
            'button_text_color',
            [
                'label' => _dmelem('Textfarbe'),
                'type' => Controls_Manager::COLOR,
                'default' => '',
                'selectors' => [
                    '{{WRAPPER}} .elementor-button' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'button_typography',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_ACCENT,
                ],
                'selector' => '{{WRAPPER}} .elementor-button',
            ]
        );

        $this->add_control(
            'button_background_color',
            [
                'label' => _dmelem('Hintergrundfarbe'),
                'type' => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_ACCENT,
                ],
                'selectors' => [
                    '{{WRAPPER}} .elementor-button' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name' => 'button_border',
                'selector' => '{{WRAPPER}} .elementor-button',
                'separator' => 'before',
            ]
        );

        $this->add_control(
            'button_border_radius',
            [
                'label' => _dmelem('Rahmenrundung'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', '%'],
                'selectors' => [
                    '{{WRAPPER}} .elementor-button' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'button_text_padding',
            [
                'label' => _dmelem('Textabstand'),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => ['px', 'em', '%'],
                'selectors' => [
                    '{{WRAPPER}} .elementor-button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'tab_button_hover',
            [
                'label' => _dmelem('Schwebend'),
            ]
        );

        $this->add_control(
            'button_hover_color',
            [
                'label' => _dmelem('Textfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-button:hover' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'button_background_hover_color',
            [
                'label' => _dmelem('Hintergrundfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-button:hover' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'button_hover_border_color',
            [
                'label' => _dmelem('Rahmenfarbe'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-button:hover' => 'border-color: {{VALUE}};',
                ],
                'condition' => [
                    'button_border_border!' => '',
                ],
            ]
        );

        $this->add_control(
            'button_hover_animation',
            [
                'label' => _dmelem('Effekt'),
                'type' => Controls_Manager::HOVER_ANIMATION,
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();
    }

    private function _register_general_controls()
    {
        $this->start_controls_section(
            'section_general',
            [
                'label' => _dmelem('Allgemein'),
                'active' => true,
            ]
        );

        $this->add_control(
            'products',
            [
                'label' => _dmelem('Produkte'),
                'type' => Controls_Manager::SELECT2,
                'multiple' => true,
                'options' => $this->get_products(),
            ]
        );

        $this->add_control(
            'show_labels',
            [
                'label' => _dmelem('Labels anzeigen'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'auto_login',
            [
                'label' => _dmelem('Automatischer Login'),
                'type' => Controls_Manager::SWITCHER,
                'default' => '',
            ]
        );

        $this->add_control(
            'hide_after_signup',
            [
                'label' => _dmelem('Formular nach dem Registrieren ausblenden'),
                'type' => Controls_Manager::SWITCHER,
                'label_block' => true,
                'default' => '',
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_fields',
            [
                'label' => _dmelem('Felder'),
            ]
        );

        $this->add_control(
            'input_size',
            [
                'label' => __('Input Size', 'elementor-pro'),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'xs' => __('Extra Small', 'elementor-pro'),
                    'sm' => __('Small', 'elementor-pro'),
                    'md' => __('Medium', 'elementor-pro'),
                    'lg' => __('Large', 'elementor-pro'),
                    'xl' => __('Extra Large', 'elementor-pro'),
                ],
                'default' => 'sm',
            ]
        );

        $this->start_controls_tabs('tabs_fields');
        $this->start_controls_tab(
            'tab_email',
            [
                'label' => _dmelem('E-Mail'),
            ]
        );

        $this->add_control(
            'label_email',
            [
                'label' => _dmelem('Label'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('E-Mail Adresse'),
                'condition' => [
                    'show_labels' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'placeholder_email',
            [
                'label' => _dmelem('Platzhalter'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('Deine E-Mail Adresse'),
            ]
        );

        $this->end_controls_tab();
        $this->start_controls_tab(
            'tab_first_name',
            [
                'label' => _dmelem('Vorname'),
            ]
        );

        $this->add_control(
            'show_first_name',
            [
                'label' => _dmelem('Vorname'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'label_first_name',
            [
                'label' => _dmelem('Label'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('Vorname'),
                'condition' => [
                    'show_labels' => 'yes',
                    'show_first_name' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'placeholder_first_name',
            [
                'label' => _dmelem('Platzhalter'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('Dein Vorname'),
                'condition' => [
                    'show_first_name' => 'yes',
                ],
            ]
        );

        $this->end_controls_tab();
        $this->start_controls_tab(
            'tab_last_name',
            [
                'label' => _dmelem('Nachname'),
            ]
        );
        
        $this->add_control(
            'show_last_name',
            [
                'label' => _dmelem('Nachname'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'label_last_name',
            [
                'label' => _dmelem('Label'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('Nachname'),
                'condition' => [
                    'show_labels' => 'yes',
                    'show_last_name' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'placeholder_last_name',
            [
                'label' => _dmelem('Platzhalter'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('Dein Nachname'),
                'condition' => [
                    'show_last_name' => 'yes',
                ],
            ]
        );

        $this->end_controls_tab();
        $this->start_controls_tab(
            'tab_checkbox',
            [
                'label' => _dmelem('Checkbox'),
            ]
        );

        $this->add_control(
            'require_checkbox',
            [
                'label' => _dmelem('Checkbox erforderlich'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
                'condition' => [
                    'show_checkbox' => 'yes',
                ],
            ]
            
        );

        $this->add_control(
            'show_checkbox',
            [
                'label' => _dmelem('Checkbox anzeigen'),
                'type' => Controls_Manager::SWITCHER,
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'label_checkbox',
            [
                'label' => _dmelem('Label'),
                'type' => Controls_Manager::WYSIWYG,
                'default' => _dmelem('Ich akzeptiere die <a href="#" target="_blank">Datenschutzbestimmungen</a>'),
                'condition' => [
                    'show_checkbox' => 'yes',
                ],
            ]
        );

        $this->end_controls_tab();
        $this->end_controls_tabs();

        $this->end_controls_section();

        $this->start_controls_section(
            'section_button',
            [
                'label' => _dmelem('Button'),
            ]
        );

        $this->add_control(
            'button_text',
            [
                'label' => _dmelem('Text'),
                'type' => Controls_Manager::TEXT,
                'default' => _dmelem('Anmelden'),
            ]
        );

        $this->add_control(
            'button_size',
            [
                'label' => __('Size', 'elementor-pro'),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'xs' => __('Extra Small', 'elementor-pro'),
                    'sm' => __('Small', 'elementor-pro'),
                    'md' => __('Medium', 'elementor-pro'),
                    'lg' => __('Large', 'elementor-pro'),
                    'xl' => __('Extra Large', 'elementor-pro'),
                ],
                'default' => 'sm',
            ]
        );
        
        $this->add_responsive_control(
            'align',
            [
                'label' => __('Alignment', 'elementor-pro'),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'start' => [
                        'title' => __('Left', 'elementor-pro'),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => __('Center', 'elementor-pro'),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'end' => [
                        'title' => __('Right', 'elementor-pro'),
                        'icon' => 'eicon-text-align-right',
                    ],
                    'stretch' => [
                        'title' => __('Justified', 'elementor-pro'),
                        'icon' => 'eicon-text-align-justify',
                    ],
                ],
                'prefix_class' => 'elementor%s-button-align-',
                'default' => '',
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_recaptcha',
            [
                'label' => _dmelem('Google reCAPTCHA'),
            ]
        );
        
        $this->add_control(
            'show_recaptcha',
            [
                'label' => _dmelem('reCAPTCHA verwenden'),
                'type' => Controls_Manager::SWITCHER,
                'default' => '',
            ]
        );

        $this->add_control(
            'recaptcha_key',
            [
                'label' => _dmelem('Websiteschlüssel'),
                'type' => Controls_Manager::TEXT,
                'condition' => [
                    'show_recaptcha' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'recaptcha_secret',
            [
                'label' => _dmelem('Geheimer Schlüssel'),
                'type' => Controls_Manager::TEXT,
                'condition' => [
                    'show_recaptcha' => 'yes',
                ],
            ]
        );
        
        $this->end_controls_section();
    }

    /**
     * @return array
     */
    private function get_products()
    {
        $return = [];
        foreach (digimember_listProducts() as $product) {
            $return[$product->id] = $product->name;
        }
        return $return;
    }

    private function _add_form_render_attributes()
    {
        $settings = $this->get_settings();
        
        if (!empty($settings['button_size'])) {
            $this->add_render_attribute('button', 'class', 'elementor-size-' . $settings['button_size']);
        }

        if ($settings['button_hover_animation']) {
            $this->add_render_attribute('button', 'class', 'elementor-animation-' . $settings['button_hover_animation']);
        }
        
        $first_name_id = ncore_id();
        $last_name_id = ncore_id();
        $email_id = ncore_id();
        $checkbox_id = ncore_id();
        $this->add_render_attribute([
            'wrapper' => [
                'class' => [
                    'elementor-form-fields-wrapper',
                ],
            ],
            'form' => [
                'class' => [
                    'elementor-form'
                ],
                'method' => 'POST',
                'id' => ncore_id(),
                'action' => $this->get_form_action_url(),
            ],
            'field-group' => [
                'class' => [
                    'elementor-field-type-text',
                    'elementor-field-group',
                    'elementor-column',
                    'elementor-col-100',
                ],
            ],
            'submit-group' => [
                'class' => [
                    'elementor-field-group',
                    'elementor-column',
                    'elementor-field-type-submit',
                    'elementor-col-100',
                ],
            ],
            'button' => [
                'class' => [
                    'elementor-button',
                ],
                'id' => ncore_id(),
                'name' => 'wp-submit',
                'type' => 'submit',
            ],
            'label_email' => [
                'for' => $email_id,
                'class' => 'elementor-field-label',
            ],
            'label_first_name' => [
                'for' => $first_name_id,
                'class' => 'elementor-field-label',
            ],
            'label_last_name' => [
                'for' => $last_name_id,
                'class' => 'elementor-field-label',
            ],
            'label_checkbox' => [
                'for' => $checkbox_id,
                'class' => 'elementor-field-label',
            ],
            'input_email' => [
                'type' => 'email',
                'name' => 'email',
                'id' => $email_id,
                'placeholder' => $settings['placeholder_email'],
                'value' => ncore_retrieve($_POST, 'email', ''),
                'class' => [
                    'elementor-field',
                    'elementor-field-textual',
                    'elementor-size-' . $settings['input_size'],
                ],
            ],
            'input_first_name' => [
                'type' => 'text',
                'name' => 'first_name',
                'id' => $first_name_id,
                'placeholder' => $settings['placeholder_first_name'],
                'value' => ncore_retrieve($_POST, 'first_name', ''),
                'class' => [
                    'elementor-field',
                    'elementor-field-textual',
                    'elementor-size-' . $settings['input_size'],
                ],
            ],
            'input_last_name' => [
                'type' => 'text',
                'name' => 'last_name',
                'id' => $last_name_id,
                'placeholder' => $settings['placeholder_last_name'],
                'value' => ncore_retrieve($_POST, 'last_name', ''),
                'class' => [
                    'elementor-field',
                    'elementor-field-textual',
                    'elementor-size-' . $settings['input_size'],
                ],
            ],
            'input_checkbox' => [
                'type' => 'checkbox',
                'name' => 'is_confirmed',
                'id' => $checkbox_id,
                'style' => 'margin-right: 10px',
                'class' => [
                    'elementor-field',
                    'elementor-field-checkbox',
                ],
            ],
        ]);

        if (!$settings['show_labels']) {
            $this->add_render_attribute('label', 'class', 'elementor-screen-only');
        }

        $this->add_render_attribute('field-group', 'class', 'elementor-field-required')
            ->add_render_attribute('input', 'required', true)
            ->add_render_attribute('input', 'aria-required', 'true');
    }

    public function render()
    {
        if ($this->is_posted()) {
            $this->handle_request();
        }
        $settings = $this->get_settings_for_display();

        if (count($settings['products']) == 0) {
            ?>
            <div class="error ncore_user ncore_error">
                <?php echo _dmelem('Bitte mindestens ein Produkt auswählen'); ?>
            </div>
            <?php
            return;
        }

        if (!Plugin::$instance->editor->is_edit_mode() && ncore_canAdmin()) {
            // Should display debug messages
            $messages = [];

            /** @var digimember_ProductData $model */
			$model = dm_api()->load->model('data/product');
            $product_ids = $settings['products'];

			foreach ($product_ids as $product_id)
			{
				if ($product_id === 'demo' || empty($product_id))
				{
					continue;
				}

				$product = $model->get($product_id);
				if (!$product)
				{
					$plugin_name = dm_api()->pluginDisplayName();
					$messages[] = _digi( 'Invalid %s product id: #%s', $plugin_name, $product_id );
					continue;
				}

				if ($product->status != 'published')
				{
					$messages[] = _digi( 'Product id #%s "<em>%s</em>" is not published.',$product_id, $product->name );
					continue;
				}
			}

			if (count($messages))
			{
                ?>
                <div class="error ncore_user ncore_error">
                    <?php echo join(', ', $messages); ?>
                </div>
                <?php
                return;
			}

        }

        $this->_add_form_render_attributes();
        $flashed_messages = trim(ncore_renderFlashMessages());
        if ($settings['hide_after_signup'] && $this->is_posted() && !$this->has_post_errors) {
            echo $flashed_messages;
            return;
        }
        ?>
        <form <?php echo $this->get_render_attribute_string('form'); ?>>
            <?php echo $flashed_messages; ?>
            <input type="hidden" value="dm_elem_signup" name="dm_elem" />
            <div <?php echo $this->get_render_attribute_string('wrapper'); ?>>
                <div <?php echo $this->get_render_attribute_string('error'); ?>></div>
                <div <?php echo $this->get_render_attribute_string('field-group'); ?>>
                    <?php
                    if ($settings['show_labels']) {
                        echo '<label ' . $this->get_render_attribute_string('label_email') . '>' . $settings['label_email'] . '</label>';
                    }
                    echo '<input ' . $this->get_render_attribute_string('input_email') . '>';
                    ?>
                </div>
                <?php if ($settings['show_first_name'] == 'yes') : ?>
                    <div <?php echo $this->get_render_attribute_string('field-group'); ?>>
                        <?php
                        if ($settings['show_labels']) {
                            echo '<label ' . $this->get_render_attribute_string('label_first_name') . '>' . $settings['label_first_name'] . '</label>';
                        }
                        echo '<input ' . $this->get_render_attribute_string('input_first_name') . '>';
                        ?>
                    </div>
                <?php endif; ?>
                <?php if ($settings['show_last_name'] == 'yes') : ?>
                    <div <?php echo $this->get_render_attribute_string('field-group'); ?>>
                        <?php
                        if ($settings['show_labels']) {
                            echo '<label ' . $this->get_render_attribute_string('label_last_name') . '>' . $settings['label_last_name'] . '</label>';
                        }
                        echo '<input ' . $this->get_render_attribute_string('input_last_name') . '>';
                        ?>
                    </div>
                <?php endif; ?>
                <?php if ($settings['show_checkbox'] == 'yes') : ?>
                    <div class="elementor-field-group" style="display: flex;">
                        <input <?php echo $this->get_render_attribute_string('input_checkbox'); ?> />
                        <label <?php echo $this->get_render_attribute_string('label_checkbox'); ?>><?php echo $settings['label_checkbox']; ?></label>
                    </div>
                <?php endif; ?>
                <?php echo $this->render_recaptcha(); ?>
                <div <?php echo $this->get_render_attribute_string('submit-group'); ?>>
                    <button <?php echo $this->get_render_attribute_string('button'); ?>>
                        <?php
                        if (!empty($settings['button_text'])) {
                            ?>
                            <span class="elementor-button-text"><?php echo $settings['button_text']; ?></span>
                            <?php
                        }
                        ?>
                    </button>
                </div>
            </div>
        </form>
        <?php
        if (!Plugin::$instance->editor->is_edit_mode()) {
            ?>
            <script>
                (function ($) {
                    var $form = $('<?php echo sprintf('form#%s', $this->get_render_attributes('form')['id'][0]); ?>');

                    var $checkbox = $form.find('input[name="<?php echo $this->get_render_attributes('input_checkbox')['name'][0]; ?>"]');
                    var $signupButton = $form.find('button[id="<?php echo $this->get_render_attributes('button')['id'][0]; ?>"]');
                    $signupButton
                        .attr('type', 'submit')
                        .on('click', function () {
                            if ('<?php echo $settings['require_checkbox']; ?>' === 'yes' && !$checkbox.prop('checked')) {
                                return false;
                            }
                            return true;
                        });

                    // Replace field names in error messages
                    var $errorLabel = $('.dm-alert-error .dm-alert-content label');
                    $form.find('input').each(function () {
                        var $input = $(this);
                        var name = $input.attr('name');
                        var label = $input
                            .siblings('label')
                            .clone()
                            .children()
                            .remove()
                            .end()
                            .text();
                        $errorLabel.each(function () {
                            var $label = $(this);
                            $label.html($label.html().replace(name, label));
                        });
                    });
                })(typeof ncoreJQ !== 'undefined' ? ncoreJQ : jQuery);
            </script>
            <?php
        }
    }

    protected function content_template()
    {
        ?>
        <div class="elementor-form" style="position: relative;">
            <#
            if (settings.products.length === 0) {
                #>
                <div style="position: absolute; top: 0; left: 0; width: 100%; height: 100%; background-color: rgba(0, 0, 0, 0.6); display: flex; justify-content: center; align-items: center; z-index: 999;">
                    <div style="padding: 40px; border-radius: 10px; background-color: rgba(255, 255, 255, 0.6);">
                        <?php echo _dmelem('Bitte mindestens ein Produkt auswählen'); ?>
                    </div>
                </div>
                <#
            }
            #>
            <div class="elementor-form-fields-wrapper">
                <div class="elementor-field-group elementor-column elementor-col-100 elementor-field-type-text">
                    <#
                    if (settings.show_labels) {
                        #>
                        <label class="elementor-field-label" for="dm_signup_email">
                            {{ settings.label_email }}
                        </label>
                        <#
                    }
                    #>
                    <input type="email" name="email" id="dm_signup_email"
                           placeholder="{{ settings.placeholder_email }}"
                           class="elementor-field elementor-field-textual elementor-size-{{ settings.input_size }}"/>
                </div>
                <#
                if (settings.show_first_name == 'yes') {
                    #>
                    <div class="elementor-field-group elementor-column elementor-col-100 elementor-field-type-text">
                        <#
                        if (settings.show_labels) {
                            #>
                            <label class="elementor-field-label" for="dm_signup_last_name">
                                {{ settings.label_first_name }}
                            </label>
                            <#
                        }
                        #>
                        <input type="text" name="first_name" id="dm_signup_firstname"
                               placeholder="{{ settings.placeholder_first_name }}"
                               class="elementor-field elementor-field-textual elementor-size-{{ settings.input_size }}"/>
                    </div>
                    <#
                }
                if (settings.show_last_name == 'yes') {
                    #>
                    <div class="elementor-field-group elementor-column elementor-col-100 elementor-field-type-text">
                        <#
                        if (settings.show_labels) {
                            #>
                            <label class="elementor-field-label" for="dm_signup_last_name">
                                {{ settings.label_last_name }}
                            </label>
                            <#
                        }
                        #>
                        <input type="text" name="last_name" id="dm_signup_last_name"
                               placeholder="{{ settings.placeholder_last_name }}"
                               class="elementor-field elementor-field-textual elementor-size-{{ settings.input_size }}"/>
                    </div>
                    <#
                }
                if (settings.show_checkbox == 'yes') {
                    #>
                    <div class="elementor-field-group" style="display: flex;">
                        <input type="checkbox" name="checkbox" id="dm_signup_checkbox" style="margin-right: 10px;" />
                        <label for="dm_signup_checkbox">{{{ settings.label_checkbox }}}</label>
                    </div>
                    <#
                }
                #>
                <div class="elementor-field-group elementor-column elementor-field-type-submit elementor-col-100">
                    <button type="submit" class="elementor-button elementor-size-{{ settings.button_size }}">
                        <span class="elementor-button-text">{{ settings.button_text }}</span>
                    </button>
                </div>
            </div>
        </div>
        <?php
    }

    private function render_recaptcha()
    {
        $settings = $this->get_settings_for_display();

        $show_recaptcha = $settings['show_recaptcha'];
        $recaptcha_key = $settings['recaptcha_key'];
        $recaptcha_secret = $settings['recaptcha_secret'];
        if ($show_recaptcha == 'yes' && trim($recaptcha_key && $recaptcha_secret)) {
            /** @var ncore_HtmlLogic $html */
            $html = dm_api()->load->model('logic/html');
            $html->includeJs('https://www.google.com/recaptcha/api.js?onload=ncoreCaptchaCallback&render=explicit');
            $fct = 'function ncoreCaptchaCallback() {
        ncoreJQ(".ncore_repatcha").each(function(i,o) {
            var id=ncoreJQ(o).attr("id");
            grecaptcha.render(id, {
              "sitekey" : "' . $recaptchaKey . '",
            });
        });
    }';
            $html->jsFunction($fct);

            return '<div style="width: 100%; display: flex; justify-content: center;" class="ncore_repatcha" id="' . ncore_id() . '"></div>';
        }
        return '';
    }

    /**
     * @return string
     */
    private function get_form_action_url()
    {
        dm_api()->load->helper('url');
        return ncore_currentUrl();
    }
} 
